package de.informatics4kids.hopfield;

import java.awt.image.BufferedImage;

/**
 * Hilfsklasse, die die Konversion von einem ein Bild repsentierenden Vektor in
 * selbiges (als Bild-Klasse) umwandelt, damit es dargestellt werden kann.
 */
public class ConversionUtils {

	/**
	 * Konvertiert einen Vektor zu einem Bild.
	 *
	 * @param v
	 *            - Der zu konvertierende Vektor
	 * @return Das konvertierte Bild
	 */
	public static BufferedImage vectorToImage(MathVector v) {
		int size = (int) Math.sqrt(v.dimension());
		BufferedImage res = new BufferedImage(size, size, BufferedImage.TYPE_INT_ARGB);
		for (int i = 0; i < v.tuple.length; i++)
			res.setRGB(i / size, i % size, binToColor(v.tuple[i]));
		return res;
	}

	/**
	 * Wandelt einen binren Wert (-1 oder 1) in den entsprechenden Farbwert um,
	 * welcher zur Darstellung des Binrwertes verwendet werden soll. Das
	 * Ergebnis orientiert sich an der Farbdefinition der Klasse
	 * {@link HopfieldApplet}.
	 *
	 * @param binValue
	 *            - Der Binrwert. Wenn etwas anderes als -1 oder 1 angenommen
	 *            wird, wird der Wert automatisch als nicht gesetzt (-1)
	 *            angesehen und dementsprechend behandelt.
	 * @return - Der zugehrige Farbwert
	 */
	private static int binToColor(int binValue) {
		return binValue == 1 ? HopfieldApplet.CELL_SET.getRGB()
				: HopfieldApplet.CELL_NOT_SET.getRGB();
	}

}
