package de.informatics4kids.hopfield;

public class MathMatrix {

	/**
	 * Reihen der Matrix
	 */
	int rows;

	/**
	 * Spalten der Matrix
	 */
	int columns;

	/**
	 * Werte der Matrix.
	 */
	int[][] matrix;

	/**
	 * Dieser sog. Kontruktor wird bei der Erstellung der Matrix einmal
	 * automatisch aufgerufen, um die Matrix zu konstruieren.
	 */
	public MathMatrix(int rows, int columns) {
		this.rows = rows;
		this.columns = columns;
		matrix = new int[rows][columns];
	}

	/**
	 * Erstellt einen Vektor mit der Werten der Matrix aus der gegebenen Zeile.
	 *
	 * @param row
	 *            - Die spezifische Zeile
	 * @return Der Zeilenvektor
	 */
	public MathVector rowVector(int row) {
		MathVector v = new MathVector(columns);
		for (int j = 0; j < columns; j++)
			v.tuple[j] = matrix[row][j];
		return v;
	}

	/**
	 * Erstellt einen Vektor mit der Werten der Matrix aus der gegebenen Spalte.
	 *
	 * @param column
	 *            - Die spezifische Spalte
	 * @return Der Spaltenvektor
	 */
	public MathVector columnVector(int column) {
		MathVector v = new MathVector(rows);
		for (int i = 0; i < rows; i++)
			v.tuple[i] = matrix[i][column];
		return v;
	}

	/**
	 * Errechnet das Matrix-Vektor-Produkt dieser Matrix mit dem gegebenen
	 * Vektor. Das Ergebnis ist wieder ein Vektor.
	 *
	 * @param vector
	 *            - Der Vektor, mit dem multipliziert werden soll
	 * @return Das Matrix-Vektor-Produkt
	 */
	public MathVector vectorProduct(MathVector vector) {
		if (rows != vector.dimension()) { // Test, ob Produkt definiert ist
			System.err.println("Vektor hat falsche Gre!");
			return null;
		}
		MathVector result = new MathVector(vector.dimension());
		for (int i = 0; i < rows; i++) {
			MathVector row = rowVector(i);
			int scalarProduct = row.scalarProduct(vector);
			result.tuple[i] = scalarProduct;
		}
		return result;
	}

	/**
	 * Addiert die gegebene Matrix zu dieser dazu.
	 *
	 * @param other
	 *            - Die dazuzuaddierende Matrix
	 */
	public void add(MathMatrix other) {
		// Auf identische Gre prfen
		if (rows != other.rows || columns != other.columns) {
			System.err.println("Matrix hat falsche Gre!");
			return;
		}
		// addiere jeden Wert auf den dazugehrenden eigenen
		for (int i = 0; i < rows; i++)
			for (int j = 0; j < columns; j++)
				matrix[i][j] += other.matrix[i][j];
	}

	/**
	 * Subtrahiert die gegebene Matrix von dieser.
	 *
	 * @param other
	 *            - Die abzuziehende Matrix
	 */
	public void subtract(MathMatrix other) {
		// Auf identische Gre prfen
		if (rows != other.rows || columns != other.columns) {
			System.err.println("Matrix hat falsche Gre!");
			return;
		}
		// ziehe jeden Wert von dem dazugehrenden eigenen ab
		for (int i = 0; i < rows; i++)
			for (int j = 0; j < columns; j++)
				matrix[i][j] -= other.matrix[i][j];
	}

}
